#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = """
Created on 27/04/2019

@author: cnheider
"""

from abc import ABCMeta, abstractmethod
from collections import Counter, deque
from draugr.writers.mixins.scalar_writer_mixin import ScalarWriterMixin

__all__ = ["Writer", "global_writer", "set_global_writer"]

from itertools import cycle
from typing import Any, Iterable, Optional

from warg import is_none_or_zero_or_negative_or_mod_zero
from warg import Number, drop_unused_kws


class Writer(ScalarWriterMixin, metaclass=ABCMeta):
    """ """

    @drop_unused_kws
    def __init__(
        self,
        *,
        interval: Optional[int] = 1,
        filters: Iterable = None,
        verbose: bool = False
    ):
        """

        :param interval:
        :param filters:
        :param verbose:"""
        self._counter = Counter()

        self._interval = interval
        self.filters = filters
        self._verbose = verbose

    def filter(self, tag: str) -> bool:
        """

            returns a boolean  value, true if to be included, False if to be excluded

            tag is in filter if not None
            and within interval for inclusion

        :param tag:
        :type tag:
        :return:
        :rtype:"""
        is_in_filters = self.filters is None or tag in self.filters
        at_interval = is_none_or_zero_or_negative_or_mod_zero(
            self._interval, self._counter[tag]
        )
        return is_in_filters and at_interval

    def __enter__(self):
        global GLOBAL_WRITER_STACK, GLOBAL_WRITER
        GLOBAL_WRITER_STACK.appendleft(self)
        GLOBAL_WRITER = self
        return self._open()

    def __exit__(self, exc_type, exc_val, exc_tb):
        global GLOBAL_WRITER, GLOBAL_WRITER_STACK

        if len(GLOBAL_WRITER_STACK) > 0:
            GLOBAL_WRITER_STACK.popleft()  # pop self

        if len(GLOBAL_WRITER_STACK) > 0:
            GLOBAL_WRITER = GLOBAL_WRITER_STACK.popleft()  # then previous
        else:
            GLOBAL_WRITER = None
        return self._close(exc_type, exc_val, exc_tb)

    def close(self) -> Any:
        """ """
        self._close()

    def open(self) -> Any:
        """ """
        self._open()

    @abstractmethod
    def _close(self, exc_type=None, exc_val=None, exc_tb=None):
        raise NotImplementedError

    @abstractmethod
    def _open(self):
        return self

    def __call__(self, *args, **kwargs):
        self.scalar(*args, *kwargs)


GLOBAL_WRITER_STACK = deque()
GLOBAL_WRITER = None


def global_writer() -> Optional[Writer]:
    """

    :return:
    :rtype:"""
    global GLOBAL_WRITER
    return GLOBAL_WRITER


def set_global_writer(writer: Writer) -> None:
    """

    :return:
    :rtype:"""
    global GLOBAL_WRITER
    # if GLOBAL_WRITER:
    # GLOBAL_WRITER_STACK TODO: push to stack if existing?

    GLOBAL_WRITER = writer



import queue
import threading

__all__ = ["Skipper"]


class Skipper:
    def __init__(self, fun):
        self.Q = queue.Queue(1)
        self.fun = fun
        self._stop = False
        self.thread = threading.Thread(target=self.loop)
        self.thread.start()

    def loop(self):
        while True:
            if self._stop:
                break
            (args, kwargs) = self.Q.get()
            if self._stop:
                break
            self.fun(*args, **kwargs)

    def stop(self):
        self._stop = True
        try:
            self.Q.put_nowait((None, None))
        except:
            pass
        self.thread.join()

    def __call__(self, *args, **kwargs):
        while not self.Q.empty():
            self.Q.get_nowait()
        self.Q.put((args, kwargs))


if __name__ == "__main__":

    def yhhgsady():
        import time

        def worker(wid):
            print(f"Worker {wid} Start")
            time.sleep(1)
            print(f"Worker {wid} END")

        S = Skipper(worker)
        S(1)
        time.sleep(0.1)
        S(2)
        time.sleep(0.1)
        S(3)
        time.sleep(4)
        S(4)
        time.sleep(0.2)
        S(5)
        time.sleep(0.2)
        S(6)
        time.sleep(0.2)
        S(7)
        time.sleep(4)
        S.stop()

    yhhgsady()



from warg import Number
from abc import ABC, abstractmethod
from itertools import cycle

__all__ = ["ScalarWriterMixin"]


class ScalarWriterMixin(ABC):
    @abstractmethod
    def _scalar(self, tag: str, value: float, step: int):
        raise NotImplementedError

    def __init__(self):
        self._blip_values = iter(cycle(range(2)))

    def scalar(self, tag: str, value: Number, step_i: int = None) -> None:
        """

        :param tag:
        :type tag:
        :param value:
        :type value:
        :param step_i:
        :type step_i:"""
        if step_i:
            if self.filter(tag):
                self._scalar(tag, value, self._counter[tag])
            self._counter[tag] = step_i
        else:
            if self.filter(tag):
                self._scalar(tag, value, self._counter[tag])
            self._counter[tag] += 1

    def blip(self, tag: str, step_i: int = None) -> None:
        """

        :param tag:
        :type tag:
        :param step_i:
        :type step_i:"""
        if step_i:
            self.scalar(tag, next(self._blip_values), step_i)
            self.scalar(tag, next(self._blip_values), step_i)
        else:
            self.scalar(tag, next(self._blip_values))
            self.scalar(tag, next(self._blip_values), self._counter[tag])



import win32gui


__all__ = ["list_window_names", "get_inner_windows", "find_all_windows"]

verbose = False


def list_window_names():
    result = []

    def win_enum_handler(hwnd, ctx):
        if win32gui.IsWindowVisible(hwnd):
            if verbose:
                print(hex(hwnd), f'"{str(win32gui.GetWindowText(hwnd))}"')
            result.append(win32gui.GetWindowText(hwnd))

    win32gui.EnumWindows(win_enum_handler, None)
    return result


def get_inner_windows(whndl):
    def callback(hwnd, hwnds):
        if win32gui.IsWindowVisible(hwnd) and win32gui.IsWindowEnabled(hwnd):
            hwnds[win32gui.GetClassName(hwnd)] = hwnd
        return True

    hwnds = {}
    win32gui.EnumChildWindows(whndl, callback, hwnds)
    return hwnds


def find_all_windows(name):
    result = []

    def win_enum_handler(hwnd, ctx):
        if win32gui.IsWindowVisible(hwnd) and win32gui.GetWindowText(hwnd) == name:
            result.append(hwnd)

    win32gui.EnumWindows(win_enum_handler, None)
    return result


if __name__ == "__main__":
    print(get_inner_windows(find_all_windows(list_window_names()[-1])[-1]))



from pathlib import Path
from typing import Iterable

import cv2
import numpy

from draugr.opencv_utilities.color_space.threshold import hsv_min_max_clip_mask
from draugr.opencv_utilities.windows.image import show_image
from draugr.opencv_utilities.windows.elements import add_trackbar
from draugr.opencv_utilities.color_space import to_gray

__all__ = ["interactive_hough_circles"]


def interactive_hough_circles(
    ps: Iterable, wait_time: int = 33, verbose: bool = False
) -> None:

    # show_image(numpy.zeros((600, 600)), "canny")

    # add_trackbar("canny", "canny_t1", default=30, max_val=1000, min_val=1.0)
    # add_trackbar("canny", "canny_t2", default=100, max_val=1000, min_val=1.0)

    show_image(numpy.zeros((600, 600)), "image")

    add_trackbar("image", "dp", default=10.0, max_val=30.0, min_val=1.0)
    add_trackbar("image", "minDist", default=4, max_val=255, min_val=1.0)
    add_trackbar(
        "image", "param1", default=50, max_val=1000, min_val=1.0
    )  # param1 (passed as argument into cvHoughCircles()) and the lower (second) threshold is set to half of this value.
    add_trackbar(
        "image", "param2", default=30, max_val=255, min_val=1.0
    )  # accumulator threshold. This value is used in the accumulator plane that must be reached so that a line is retrieved.
    add_trackbar("image", "minRadius", default=0, max_val=255, min_val=1.0)
    add_trackbar("image", "maxRadius", default=255, max_val=255, min_val=1.0)

    canny_t1 = canny_t2 = dp = minDist = param1 = param2 = minRadius = maxRadius = 0
    pcanny_t1 = (
        pcanny_t2
    ) = pdp = pminDist = pparam1 = pparam2 = pminRadius = pmaxRadius = 0

    for p in ps:
        if not p.exists():
            continue

        img = cv2.imread(str(p))
        while 1:
            output = img.copy()

            # canny_t1 = cv2.getTrackbarPos("canny_t1", "canny")
            # canny_t2 = cv2.getTrackbarPos("canny_t2", "canny")

            dp = cv2.getTrackbarPos("dp", "image") / 10
            minDist = cv2.getTrackbarPos("minDist", "image")
            param1 = cv2.getTrackbarPos("param1", "image")
            param2 = cv2.getTrackbarPos("param2", "image")
            minRadius = cv2.getTrackbarPos("minRadius", "image")
            maxRadius = cv2.getTrackbarPos("maxRadius", "image")

            # imgray = cv2.Canny(output, canny_t1, canny_t2)
            imgray = to_gray(output)
            circles = cv2.HoughCircles(
                imgray,
                method=cv2.HOUGH_GRADIENT,
                dp=dp,
                minDist=minDist,
                param1=param1,
                param2=param2,
                minRadius=minRadius,
                maxRadius=maxRadius,
            )

            if circles is not None:
                circles = numpy.uint16(numpy.around(circles))
                blue = (0, 0, 255)
                green = (0, 255, 0)
                for c in circles[0]:
                    cv2.circle(output, (c[0], c[1]), c[2], blue, 3)

            # Print if there is a change in HSV value
            if (
                (pcanny_t1 != canny_t1)
                | (pcanny_t2 != canny_t2)
                | (pdp != dp)
                | (pminDist != minDist)
                | (pparam1 != param1)
                | (pparam2 != param2)
                | (pminRadius != minRadius)
                | (pmaxRadius != maxRadius)
            ):

                # cv2.setTrackbarPos("canny_t1", "canny", canny_t1)
                # cv2.setTrackbarPos("canny_t2", "canny", canny_t2)

                cv2.setTrackbarPos("dp", "image", int(dp * 10))
                cv2.setTrackbarPos("minDist", "image", minDist)
                cv2.setTrackbarPos("param1", "image", param1)
                cv2.setTrackbarPos("param2", "image", param2)
                cv2.setTrackbarPos("minRadius", "image", minRadius)
                cv2.setTrackbarPos("maxRadius", "image", maxRadius)

                pcanny_t1 = canny_t1
                pcanny_t1 = canny_t1
                pdp = dp
                pminDist = minDist
                pparam1 = param1
                pparam2 = param2
                pminRadius = minRadius
                pmaxRadius = maxRadius

            # show_image(imgray, "canny")

            if show_image(output, "image", wait=wait_time):
                break

    cv2.destroyAllWindows()


if __name__ == "__main__":
    pss = (
        Path.home()
        / "ProjectsWin/AiBitbucket/Internal/OptikosPrime/exclude/new_images/adam_plus0p5/215asd.jpg",
        Path(r"C:\Users\Christian\OneDrive\Billeder\buh\7BIsT.png"),
        Path(r"C:\Users\Christian\OneDrive\Billeder\Portraits\thomas.jpg"),
    )
    if any(p.exists() for p in pss):
        interactive_hough_circles(iter(pss))
    else:
        print(f"{pss} does not exist")



from enum import Enum

__all__ = ["HttpMethodEnum"]


class HttpMethodEnum(Enum):
    get = "GET"
    post = "POST"
    head = "HEAD"
    put = "PUT"
    delete = "DELETE"
    connect = "CONNECT"
    options = "OPTIONS"
    trace = "TRACE"
    patch = "PATCH"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from typing import IO, List, Sequence, Union


def python_version_check(major: int = 3, minor: int = 7):
    """ """
    import sys

    assert sys.version_info.major == major and sys.version_info.minor >= minor, (
        f"This project is utilises language features only present Python {major}.{minor} and greater. "
        f"You are running {sys.version_info}."
    )


python_version_check()

from pathlib import Path
import re

from setuptools import find_packages, setup


def read_reqs(file: str, path: Path) -> List[str]:
    """ """

    def readlines_ignore_comments(f: IO):
        """ """
        return [a_ for a_ in f.readlines() if "#" not in a_ and a_]

    def recursive_flatten_ignore_str(seq: Sequence) -> Sequence:
        """ """
        if not seq:  # is empty Sequence
            return seq
        if isinstance(seq[0], str):
            return seq
        if isinstance(seq[0], Sequence):
            return (
                *recursive_flatten_ignore_str(seq[0]),
                *recursive_flatten_ignore_str(seq[1:]),
            )
        return (*seq[:1], *recursive_flatten_ignore_str(seq[1:]))

    def unroll_nested_reqs(req_str: str, base_path: Path) -> Sequence:
        """ """
        if req_str.startswith("-r"):
            with open(base_path / req_str.strip("-r").strip()) as f:
                return [
                    unroll_nested_reqs(req.strip(), base_path)
                    for req in readlines_ignore_comments(f)
                ]
        else:
            return (req_str,)

    requirements_group = []
    with open(str(path / file)) as f:
        requirements = readlines_ignore_comments(f)
        for requirement in requirements:
            requirements_group.extend(
                recursive_flatten_ignore_str(
                    unroll_nested_reqs(requirement.strip(), path)
                )
            )

    req_set = set(requirements_group)
    req_set.discard("")
    return list(req_set)


with open(Path(__file__).parent / "draugr" / "__init__.py", "r") as project_init_file:
    content = project_init_file.read()  # get strings from module
    version = re.search(r"__version__ = ['\"]([^'\"]*)['\"]", content, re.M).group(1)
    project_name = re.search(r"__project__ = ['\"]([^'\"]*)['\"]", content, re.M).group(
        1
    )
    author = re.search(r"__author__ = ['\"]([^'\"]*)['\"]", content, re.M).group(1)

__author__ = author


class DraugrPackage:
    @property
    def test_dependencies(self) -> list:
        return read_reqs(
            "requirements_tests.txt", Path(__file__).parent / "requirements"
        )

    @property
    def setup_dependencies(self) -> list:
        """ """
        return read_reqs(
            "requirements_setup.txt", Path(__file__).parent / "requirements"
        )

    @property
    def package_name(self) -> str:
        """ """
        return project_name

    @property
    def url(self) -> str:
        """ """
        return "https://github.com/cnheider/draugr"

    @property
    def download_url(self) -> str:
        """ """
        return f"{self.url}/releases"

    @property
    def readme_type(self) -> str:
        """ """
        return "text/markdown"

    @property
    def packages(self) -> List[Union[bytes, str]]:
        """ """
        return find_packages(
            exclude=[
                # 'Path/To/Exclude'
            ]
        )

    @property
    def author_name(self) -> str:
        """ """
        return author

    @property
    def author_email(self) -> str:
        """ """
        return "christian.heider@alexandra.dk"

    @property
    def maintainer_name(self) -> str:
        """ """
        return self.author_name

    @property
    def maintainer_email(self) -> str:
        """ """
        return self.author_email

    @property
    def package_data(self) -> dict:
        """ """
        emds = [str(p) for p in Path(__file__).parent.rglob(".md")]
        return {"draugr": [*emds]}

    @property
    def entry_points(self) -> dict:
        """ """
        return {
            "console_scripts": [
                # "name_of_executable = module.with:function_to_execute"
                "draugr-darkmode-toggle = draugr.entry_points.toggle_darkmode:main",
                "draugr-tb = draugr.entry_points.tensorboard_entry_point:main",
                "draugr-cpu = draugr.entry_points.cpu_usage_entry_point:main",
                "draugr-vis = draugr.entry_points.visdom_entry_point:main",
            ]
        }

    @property
    def extras(self) -> dict:
        """ """
        these_extras = {
            # 'ExtraName':['package-name; platform_system == "System(Linux,Windows)"'
        }

        path: Path = Path(__file__).parent / "requirements"

        for file in path.iterdir():
            if file.name.startswith("requirements_"):
                group_name_ = "_".join(file.name.strip(".txt").split("_")[1:])
                these_extras[group_name_] = read_reqs(file.name, path)

        all_dependencies = []
        for group_name in these_extras:
            all_dependencies += these_extras[group_name]
        these_extras["all"] = list(set(all_dependencies))

        return these_extras

    @property
    def requirements(self) -> list:
        """ """
        return read_reqs("requirements.txt", Path(__file__).parent)

    @property
    def description(self) -> str:
        """ """
        return "A package for plotting directly in your terminal"

    @property
    def readme(self) -> str:
        """ """
        with open("README.md", encoding="utf8") as f:
            return f.read()

    @property
    def changelog(self) -> str:
        """ """
        with open("CHANGELOG.md") as f:
            return f.read()

    @property
    def keyword(self) -> str:
        """ """
        with open("KEYWORDS.md") as f:
            return f.read()

    @property
    def license(self) -> str:
        """ """
        return "Apache License, Version 2.0"

    @property
    def classifiers(self) -> List[str]:
        """ """
        return [
            "Development Status :: 4 - Beta",
            "Environment :: Console",
            "Intended Audience :: End Users/Desktop",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: Apache Software License",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows",
            "Operating System :: POSIX",
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 3",
            "Natural Language :: English",
            # 'Topic :: Scientific/Engineering :: Artificial Intelligence'
            # 'Topic :: Software Development :: Bug Tracking',
        ]

    @property
    def version(self) -> str:
        """ """
        return version


if __name__ == "__main__":
    pkg = DraugrPackage()

    setup(
        name=pkg.package_name,
        version=pkg.version,
        packages=pkg.packages,
        package_data=pkg.package_data,
        author=pkg.author_name,
        author_email=pkg.author_email,
        maintainer=pkg.maintainer_name,
        maintainer_email=pkg.maintainer_email,
        description=pkg.description,
        license=pkg.license,
        keywords=pkg.keyword,
        url=pkg.url,
        download_url=pkg.download_url,
        install_requires=pkg.requirements,
        extras_require=pkg.extras,
        setup_requires=pkg.setup_dependencies,
        entry_points=pkg.entry_points,
        classifiers=pkg.classifiers,
        long_description_content_type=pkg.readme_type,
        long_description=pkg.readme,
        tests_require=pkg.test_dependencies,
        include_package_data=True,
        python_requires=">=3.6",
    )
